// -*- tab-width:4 -*-
#pragma once
#if !defined(ALFSTREAM_HPP_)
#define ALFSTREAM_HPP_

/**@file
   @brief alfstream/alifstream/alofstream クラスの宣言。
 **/

#include <iostream>
#include "alstdiofilebuf.hpp"

namespace al {
namespace file {

/**
   基底テンプレートクラスの宣言。
 **/
template<typename _baseclassT, typename _CharT,typename _Traits,std::ios_base::openmode BASE_OPENMODE>
class basic_alfstream
    :	public alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>,
        public _baseclassT
{
public:
    basic_alfstream() throw(std::bad_alloc)
    : alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>(), _baseclassT() {}
    
    basic_alfstream(const char* __path/*UTF-8*/,
                    std::ios_base::openmode __openmode=static_cast<std::ios_base::openmode>(0),
                    bool __make_read_only=false )
        throw(std::bad_alloc,std::ios_base::failure)
        : alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>(__path,__openmode,__make_read_only),
          _baseclassT(alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::rdbuf()) {
        _baseclassT::imbue(std::locale());
        if( !alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::is_open() )
            _baseclassT::setstate(std::ios_base::badbit);
    }

    basic_alfstream(const wchar_t* __path/*UTF-16*/,
                    std::ios_base::openmode __openmode=static_cast<std::ios_base::openmode>(0),
                    bool __make_read_only=false)
        throw(std::bad_alloc,std::ios_base::failure)
        : alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>(__path,__openmode,__make_read_only),
          _baseclassT(alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::rdbuf()) {
        _baseclassT::imbue(std::locale());
        if( !_baseclassT::is_open() )
            _baseclassT::setstate(std::ios_base::badbit);
    }

    virtual ~basic_alfstream() {}

    template<typename _filenameCharT>
    void open(_filenameCharT* __path,
              std::ios_base::openmode __openmode=static_cast<std::ios_base::openmode>(0),
              bool __make_read_only=false)
        throw(std::bad_alloc,std::ios_base::failure)
    {
        try {
            alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::open(__path,__openmode,__make_read_only);
            if( !alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::is_open() )
                throw std::ios_base::failure("open error");

            _baseclassT::rdbuf(alstdiofilebuf<_CharT,_Traits,BASE_OPENMODE>::rdbuf());
            _baseclassT::imbue(std::locale());
        }
        catch( std::ios_base::failure ) {
            _baseclassT::setstate(_baseclassT::badbit);
            if( _baseclassT::exceptions() & _baseclassT::badbit )
                throw;
        }
        catch( std::bad_alloc ) {
            _baseclassT::setstate(_baseclassT::failbit);
            if( _baseclassT::exceptions() & _baseclassT::failbit )
                throw;
        }
    }

};

typedef basic_alfstream<std::iostream,char,std::char_traits<char>,
                        static_cast<std::ios_base::openmode>(0)>	alfstream;
typedef basic_alfstream<std::istream,char,std::char_traits<char>,
                        std::ios_base::in>	alifstream;
typedef basic_alfstream<std::ostream,char,std::char_traits<char>,
                        std::ios_base::out>	alofstream;

} /* namespace file */
} /* namespace al */

#endif /*ALFSTREAM_HPP_*/
