// -*- tab-width:4 -*-
/**@file
   @brief basic_fstream クラスの実装。
 **/

#include "alfstream.hpp"
#include "ALSysCharSet.hpp"

#include <fcntl.h>
#include <sys/stat.h>
#include <ext/stdio_filebuf.h>

#include <string>
#include <exception>

namespace al {
namespace file {

namespace {


}


alstdiofile::alstdiofile()
    :	_fd(-1), _c_mode(), _mode()
{
}

alstdiofile::~alstdiofile()
{
    close_file();
}

void alstdiofile::open_file(const char* __path, std::ios_base::openmode __openmode,
                            bool __make_read_only )
    throw(std::bad_alloc,std::ios_base::failure)
{
#if !defined(_LINUX)
    std::wstring	u16path;
    al::CSysCharSet::GetDefSysCharSet().ConvertToSysWCS(__path,u16path);
    open_file(u16path.c_str(),__openmode,__make_read_only);
    return;
#endif
    _mode = __openmode;
    _c_mode = c_openmode(_mode);
    if( _c_mode == ~0 )
        throw std::ios_base::failure("alstdiofile::open_file:_c_mode==~0");
    _fd = open(__path,_c_mode,(_S_IREAD | (__make_read_only ? 0 : _S_IWRITE)));
}

void alstdiofile::open_file(const wchar_t* __path, std::ios_base::openmode __openmode,
                            bool __make_read_only )
    throw(std::bad_alloc,std::ios_base::failure)
{
#if defined(_LINUX)
    std::string	u8path;
    al::CSysCharSet::GetDefSysCharSet().ConvertToSysMBCS(__path,u8path);
    open_file(u8path.c_str(),__openmode, __make_read_only);
    return;
#endif
    _mode = __openmode;
    _c_mode = c_openmode(_mode);
    if( _c_mode == ~0 )
        throw std::ios_base::failure("alstdiofile::open_file:_c_mode==~0");
    _fd = _wopen(__path,_c_mode,(_S_IREAD | (__make_read_only ? 0: _S_IWRITE)));
}

void alstdiofile::close_file()
{
    if( _fd >= 0 )
        ::close(_fd);
    _fd = -1;
}

int alstdiofile::c_openmode(std::ios_base::openmode __mode)
{
    using namespace std;
    int	c_mode = ~0;

    if( (__mode & (ios_base::in|ios_base::out)) == (ios_base::in|ios_base::out) )
        c_mode = (_O_CREAT|_O_RDWR);
    else if( (__mode & ios_base::out) )
        c_mode = (_O_CREAT|_O_WRONLY);
    else if( (__mode & ios_base::in) )
        c_mode = _O_RDONLY;

    if( c_mode != ~0 ) {
        if( (__mode & ios_base::trunc) )
            c_mode |= _O_TRUNC;
        if( (__mode & ios_base::app) )
            c_mode |= _O_APPEND;
        if( (__mode & ios_base::binary) )
            c_mode |= _O_BINARY;
    }

    return c_mode;
}


} /* namespace file */
} /* namespace al */
