// -*- tab-width:4 -*-
#pragma once
#if !defined(ALSTDIOFILE_HPP_)
#define ALSTDIOFILE_HPP_

/**@file
   @brief stdioタイプ標準入出力を使うためのヘルパクラス及び関数を定義。
**/

namespace al {
namespace file {

/**
   stdioタイプ標準入出力のためのヘルパ。<br>
   <ul>
   <li>UTF-8を窓口としたUNICODEファイル名によるファイルオープン／クローズ
   </ul>
 **/
class alstdiofile
{
    int _fd;
    int _c_mode;
    std::ios_base::openmode	_mode;

public:
    alstdiofile();
    virtual ~alstdiofile();

    void open_file(const char* __path, std::ios_base::openmode __openmode,
                   bool __make_read_only )
        throw(std::bad_alloc,std::ios_base::failure);

    void open_file(const wchar_t* __path, std::ios_base::openmode __openmode,
                   bool __make_read_only)
        throw(std::bad_alloc,std::ios_base::failure);

    void close_file();

    static int c_openmode(std::ios_base::openmode __mode);


    int fd() const { return _fd; }
    int c_mode() const { return _c_mode; }
    std::ios_base::openmode	mode() const { return _mode; }

    bool is_open() const { return (_fd >= 0); }
};


} /* namespace file */
} /* namespace al */

#endif /*ALSTDIOFILE_HPP_*/
