// -*- tab-width:4 -*-
#pragma once
#if !defined(ALSTDIOFILEBUF_HPP_)
#define ALSTDIOFILEBUF_HPP_

/**@file
   @brief alstdiofilebuf クラスの宣言。
**/

#include <exception>
#include <ext/stdio_filebuf.h>

#include "alstdiofile.hpp"


namespace al {
namespace file {

/**
   ファイルバッファ生成テンプレートクラス。
 **/

template <typename _CharT,typename _Traits,std::ios_base::openmode BASE_OPENMODE>
class alstdiofilebuf : public alstdiofile {

    __gnu_cxx::stdio_filebuf<_CharT,_Traits>*	_buf;

public:
    alstdiofilebuf() : _buf() {}

    alstdiofilebuf(const char* __filename,std::ios_base::openmode __mode,
                   bool __make_read_only)
        : _buf()
    {
        open(__filename,__mode,__make_read_only);
    }

    alstdiofilebuf(const wchar_t* __filename,std::ios_base::openmode __mode,
                   bool __make_read_only)
        : _buf()
    {
        open(__filename,__mode,__make_read_only);
    }

    virtual ~alstdiofilebuf() {
        close();
    }

    void connect_to_buffer()
        throw(std::bad_alloc,std::ios_base::failure)
    {
        _buf = new __gnu_cxx::stdio_filebuf<_CharT>(fd(),mode());
        if( _buf == 0 )
            throw std::ios_base::failure("alstdiofilebuf:_buf==0");
    }

    template<typename _filenameCharT>
    void open(_filenameCharT* __filename, std::ios_base::openmode __mode,
              bool __make_read_only)
        throw(std::bad_alloc,std::ios_base::failure)
    {
        if( (static_cast<unsigned int>(__mode)
             & static_cast<unsigned int>(std::ios_base::in|std::ios_base::out)) == 0 ) {
            if( BASE_OPENMODE == static_cast<std::ios_base::openmode>(0) )
                __mode |= (std::ios_base::in|std::ios_base::out);
            else
                __mode |= BASE_OPENMODE;
        }

        open_file(__filename,__mode,__make_read_only);
        connect_to_buffer();
    }

    void close()
    {
        if( _buf ) {
            _buf->close();
            delete _buf;
        }
        _buf = 0;

        close_file();
    }

    __gnu_cxx::stdio_filebuf<_CharT>* rdbuf() const { return _buf; }

};

} /* namespace file */
} /* namespace al */


#endif /*ALSTDIOFILEBUF_HPP_*/
